/*
 * Copyright 2008 Sun Microsystems, Inc.  All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *   - Neither the name of Sun Microsystems nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package demo.fx;

import com.sun.scenario.animation.BeanProperty;
import com.sun.scenario.animation.Property;
import com.sun.scenario.scenegraph.JSGPanel;
import com.sun.scenario.scenegraph.SGGroup;
import com.sun.scenario.scenegraph.SGShape;
import com.sun.scenario.scenegraph.SGText;
import com.sun.scenario.scenegraph.SGTransform;
import com.sun.scenario.scenegraph.fx.FXGroup;
import com.sun.scenario.scenegraph.fx.FXImage;
import com.sun.scenario.scenegraph.fx.FXNode;
import demo.util.SceneTreeView;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.GraphicsEnvironment;
import java.awt.GridLayout;
import java.awt.Panel;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.Transparency;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.Point2D;
import java.awt.image.BufferedImage;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.SwingUtilities;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * @author Flar
 */
public class TransformTest extends JSGPanel {
    public static final int CANVAS_W = 500;
    public static final int CANVAS_H = 500;
    public static final int IMG_W = 100;
    public static final int IMG_H = 100;
    public static final int BORDER = 10;
    public static final int FONT_SIZE = 24;
    public static final int TEXT_X = 15;
    public static final int TEXT_Y = 60;
    
    public static final Color BORDER_COLOR = Color.GREEN;
    public static final Color CENTER_COLOR = Color.BLUE;
    public static final Color TEXT_COLOR = Color.YELLOW;
    public static final Font TEXT_FONT = new Font("Serif", Font.PLAIN, FONT_SIZE);
    public static final String TEXT = "Txform";

    FXNode node;

    public TransformTest() {
        SGGroup g = new SGGroup();
        SGShape s = new SGShape();
        s.setShape(new Rectangle(0, 0, CANVAS_W, CANVAS_H));
        s.setFillPaint(Color.WHITE);
        g.add(s);
        if (false) {
            BufferedImage img = GraphicsEnvironment.
                getLocalGraphicsEnvironment().getDefaultScreenDevice().
                getDefaultConfiguration().
                createCompatibleImage(IMG_W, IMG_H, Transparency.TRANSLUCENT);
            Graphics2D g2 = img.createGraphics();
            g2.setColor(BORDER_COLOR);
            g2.fillRect(0, 0, IMG_W, IMG_H);
            g2.setColor(CENTER_COLOR);
            g2.fillRect(BORDER, BORDER, IMG_W-BORDER*2, IMG_H-BORDER*2);
            g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                                RenderingHints.VALUE_ANTIALIAS_ON);
            g2.setColor(TEXT_COLOR);
            g2.setFont(TEXT_FONT);
            g2.drawString(TEXT, TEXT_X, TEXT_Y);
            g2.dispose();

            FXImage imgnode = new FXImage();
            imgnode.setImage(img);
            imgnode.setLocation(new Point2D.Float());
            node = imgnode;
        } else {
            FXGroup fxg = new FXGroup();
            s = new SGShape();
            s.setShape(new Rectangle(0, 0, IMG_W, IMG_H));
            s.setFillPaint(BORDER_COLOR);
            fxg.add(s);
            s = new SGShape();
            s.setShape(new Rectangle(BORDER, BORDER, IMG_W-BORDER*2, IMG_H-BORDER*2));
            s.setFillPaint(CENTER_COLOR);
            fxg.add(s);
            SGText t = new SGText();
            t.setText(TEXT);
            t.setFont(TEXT_FONT);
            t.setFillPaint(TEXT_COLOR);
            t.setLocation(new Point2D.Float(TEXT_X, TEXT_Y));
            t.setAntialiasingHint(RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
            fxg.add(t);
            node = fxg;
        }
        g.add(SGTransform.createTranslation(CANVAS_W/2, CANVAS_H/2, node));
        setScene(g);
    }

    public Component getControlPanel() {
        Panel p = new Panel();
        p.setLayout(new GridLayout(0, 2));
        p.add(makeSlider("Translate X", -CANVAS_W/2, CANVAS_W/2, 1.0, "translateX"));
        p.add(makeSlider("Translate Y", -CANVAS_H/2, CANVAS_H/2, 1.0, "translateY"));
        //p.add(makeSlider("Anchor X", 0, IMG_W, 1.0, "anchorX")); 
        //p.add(makeSlider("Anchor Y", 0, IMG_H, 1.0, "anchorY"));
        p.add(makeSlider("Rotation", -360, 360, "-360", "360", Math.PI/180.0, "rotation"));
        p.add(makeSlider("Opacity", 0, 100, 0.01, "opacity"));
        p.add(makeSlider("Scale X", 0, 100, 0.02, "scaleX"));
        p.add(makeSlider("Scale Y", 0, 100, 0.02, "scaleY"));
        // p.add(makeSlider("Shear X", 0, 100, 0.02, "shearX"));
        // p.add(makeSlider("Shear Y", 0, 100, 0.02, "shearY"));
        return p;
    }

    public Component makeSlider(String label, int min, int max,
                                final double scale, String propname)
    {
        String minLabel = Double.toString(min*scale);
        String maxLabel = Double.toString(max*scale);
        return makeSlider(label, min, max, minLabel, maxLabel, scale, propname);
    }

    public Property<Double> getDoubleProp(String propname) {
        Property<Double> prop = null;
        try {
            prop = new BeanProperty<Double>(node, propname);
            prop.getValue().doubleValue();
        } catch (ClassCastException e) {
            return getDelegatedDoubleProp(propname);
        } catch (IllegalArgumentException iae) {
            return getDelegatedDoubleProp(propname);
        }
        return prop;
    }

    public Property<Double> getDelegatedDoubleProp(String propname) {
        Property<Double> prop;
        final BeanProperty<Float> fprop =
                new BeanProperty<Float>(node, propname);
        prop = new Property<Double>() {

            public void setValue(Double v) {
                fprop.setValue(v.floatValue());
            }

            public Double getValue() {
                return fprop.getValue().doubleValue();
            }
        };
        return prop;
    }

    public Component makeSlider(String label, int min, int max,
                                String minLabel, String maxLabel,
                                final double scale, String propname)
    {
        final Property<Double> prop = getDoubleProp(propname);
        final int initval = (int) Math.round(prop.getValue() / scale);
        final JSlider s = new JSlider(min, max, initval);
        s.addChangeListener(new ChangeListener() {
            public void stateChanged(ChangeEvent e) {
                JSlider s = (JSlider) e.getSource();
                prop.setValue(s.getValue() * scale);
            }
        });
        JButton b = new JButton("reset");
        b.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                s.setValue(initval);
            }
        });
        JPanel p = new JPanel();
        p.setLayout(new FlowLayout(FlowLayout.LEFT));
        p.add(b);
        p.add(new JLabel(label+": "+minLabel));
        p.add(s);
        p.add(new JLabel(" "+maxLabel));
        return p;
    }

    public static void main(String[] args) {
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                JFrame frame = new JFrame();
                TransformTest test = new TransformTest();
                frame.add(test, BorderLayout.CENTER);
                frame.add(test.getControlPanel(), BorderLayout.SOUTH);
                SceneTreeView stv = new SceneTreeView();
                stv.setDefaultFXNodeCollapseState(false);
                stv.setScene(test.getScene());
                stv.setRefreshInterval(100);
                stv.setPreferredSize(new Dimension(300, 300));
                frame.add(stv, BorderLayout.EAST);
                frame.pack();
                frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
                frame.setLocationRelativeTo(null);
                frame.setVisible(true);
            }
        });
    }
}
